//------------------------------------------------------------------
#property copyright "www.forex-tsd.com"
#property link      "www.forex-tsd.com"
//------------------------------------------------------------------
#property indicator_separate_window
#property indicator_buffers 2
#property indicator_color1  LimeGreen
#property indicator_color2  PaleVioletRed
#property indicator_width1  2
#property indicator_width2  2
#property indicator_minimum 0
#property indicator_maximum 1

//
//
//
//
//

extern string  TimeFrame   = "current time frame";
extern int     HmaLength1  = 14;
extern int     Price1      =  0;
extern double  Power1      =  2;
extern int     HmaLength2  = 14;
extern int     Price2      =  0;
extern double  Power2      = 0.5;
extern bool    alertsOn        = false;
extern bool    alertsOnCurrent = true;
extern bool    alertsMessage   = true;
extern bool    alertsSound     = true;
extern bool    alertsEmail     = false;

//
//
//
//
//

double hull1[];
double hull2[];
double valueUp[];
double valueDn[];
double trend[];

int    timeFrame;
string indicatorFileName;
bool   returnBars;
bool   calculateValue;

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
      SetIndexBuffer(0,valueUp); SetIndexStyle(0,DRAW_HISTOGRAM);
      SetIndexBuffer(1,valueDn); SetIndexStyle(1,DRAW_HISTOGRAM);
      SetIndexBuffer(2,hull1);
      SetIndexBuffer(3,hull2);
      SetIndexBuffer(4,trend);

      //
      //
      //
      //
      //
         
         indicatorFileName = WindowExpertName();
         calculateValue    = (TimeFrame=="calculateValue"); if (calculateValue) return(0);
         returnBars        = (TimeFrame=="returnBars");     if (returnBars)     return(0);
         timeFrame         = stringToTimeFrame(TimeFrame);

      //
      //
      //
      //
      //
   
   IndicatorShortName(timeFrameToString(timeFrame)+" Hull moving average x 2("+HmaLength1+","+HmaLength2+")");
   return(0);
}



//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { hull1[0] = limit+1; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      int HalfPeriod1 = MathFloor(HmaLength1/2);
      int HullPeriod1 = MathFloor(MathSqrt(HmaLength1));
      int HalfPeriod2 = MathFloor(HmaLength2/2);
      int HullPeriod2 = MathFloor(MathSqrt(HmaLength2));
      
      //
      //
      //
      //
      //
      
      for(int i=limit; i>=0; i--)
      {
         double price1     = iMA(NULL,0,1,0,MODE_SMA,Price1,i);
         double price2     = iMA(NULL,0,1,0,MODE_SMA,Price2,i);
                hull1[i]   = iLwmp(2.0*iLwmp(price1,HalfPeriod1,Power1,i,0)-iLwmp(price1,HmaLength1,Power1,i,1),HullPeriod1,Power1,i,2);
                hull2[i]   = iLwmp(2.0*iLwmp(price2,HalfPeriod2,Power2,i,3)-iLwmp(price2,HmaLength2,Power2,i,4),HullPeriod2,Power2,i,5);
                valueUp[i] = EMPTY_VALUE;
                valueDn[i] = EMPTY_VALUE;
                trend[i]   = trend[i+1];
                  if (hull1[i]>hull2[i]) trend[i]   =  1;
                  if (hull1[i]<hull2[i]) trend[i]   = -1;
                  if (trend[i] ==  1)    valueUp[i] =  1;
                  if (trend[i] == -1)    valueDn[i] =  1;
      }
      manageAlerts();
      return(0);
   }
   
   //
   //
   //
   //
   //
   

   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         valueUp[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HmaLength1,Price1,Power1,HmaLength2,Price2,Power2,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);
         valueDn[i] = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",HmaLength1,Price1,Power1,HmaLength2,Price2,Power2,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);
   }
   return(0);         
}





//------------------------------------------------------------------
//                                                                 
//------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if( alertsOn ) 
   {
      if (alertsOnCurrent) 
         int whichBar = 0;
      else   whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"BUY");
         if (trend[whichBar] == -1) doAlert(whichBar,"SELL");
      }
   }
}

//
//
//
//
//

void doAlert( int forBar, string doWhat )
{
   static string   previousAlert = "nothing";
   static datetime previousTime;
   string message;
   
   if( previousAlert != doWhat  ||  previousTime != Time[forBar] ) {
      previousAlert  = doWhat;
      previousTime   = Time[forBar];
      
      message =  StringConcatenate( timeFrameToString(Period())," ",Symbol()," HULL PARABOLIC ", doWhat );
      if ( alertsMessage ) Alert(message);
      if ( alertsEmail ) SendMail( StringConcatenate(Symbol(), " ", WindowExpertName(), " ", timeFrameToString(timeFrame)), message );
      if ( alertsSound ) PlaySound("alert2.wav");
   }
}

//------------------------------------------------------------------
//                                                                 
//------------------------------------------------------------------
//
//
//
//
//

double workLwmp[][6];
double iLwmp(double price, double period, double power, int r, int instanceNo=0)
{
   if (ArrayRange(workLwmp,0)!= Bars) ArrayResize(workLwmp,Bars); r = Bars-r-1;
   
   //
   //
   //
   //
   //
   
   workLwmp[r][instanceNo] = price;
      double sumw = MathPow(period,power);
      double sum  = sumw*price;
      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = MathPow(period-k,power);
                sumw  += weight;
                sum   += weight*workLwmp[r-k][instanceNo];  
      }             
   return(sum/sumw);
}

//------------------------------------------------------------------
//
//------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("M" + DoubleToStr(timeFrame,0));
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}